// uCalcCPP.h : Header file for uCalc Fast Math Parser 2.96
// Designed to work with 32-bit MS Visual C++ and Borland C++ Builder.
//
// Copyright (c) 1999-2008 by Daniel Corbier.
// Visit www.ucalc.com for additional resources.

#ifndef __UCALC__
#define __UCALC__

#pragma once

#include <wtypes.h>
#include <string.h>

// Definition properties
const long ucAlphaNumeric   = 1;
const long ucArgSeparator   = 2;
const long ucCodeBlock      = 3;
const long ucCodeBlockMatch = 4;
const long ucEvalInsert     = 5;
const long ucExpandInsert   = 6;
const long ucMemberAccess   = 7;
const long ucQuotedText     = 8;
const long ucStatementSep   = 9;
const long ucWhiteSpace     = 10;

const long ucFuncDefEnd = 11;
const long ucCurrentThread = 12;

const long ucPartialBlockStart = 13;
const long ucPartialBlockEnd   = 14;

const long ucEvalDuringParse = 15;

const long ucFunction       = 16;
const long ucLiteral        = 32;
const long ucReducible      = 64;
const long ucSymbol         = 128;
const long ucVariable       = 256;
const long ucInfix          = 512;
const long ucPrefix         = 1024;
const long ucPostfix        = 2048;
const long ucOperator       = 4096;
const long ucPassOnce       = 8192;
const long ucLock           = 16384;


// const ucalc Commands
const long uc_Define             = 1;
const long uc_DefineSyntax       = 2;
const long uc_EvalDbl            = 4;
const long uc_EvalExt            = 5;
const long uc_EvalStr            = 6;
const long uc_EvaluateDbl        = 7;
const long uc_EvaluateExt        = 8;
const long uc_Evaluate           = 8;
const long uc_EvaluateLng        = 9;
const long uc_EvaluateStr        = 10;
const long uc_GetCaseSensitivity = 11;
const long uc_GetCodeAddr        = 12;
const long uc_GetDefaultPrecedence = 13;
const long uc_GetDefaultRank     = 14;
const long uc_GetDefaultType     = 15;
const long uc_GetDefaultThread   = 16;
const long uc_GetDefSeparator    = 17;
const long uc_GetErrorNumber     = 18;
const long uc_GetParentThread    = 19;
const long uc_GetSyntaxEqual     = 20;
const long uc_Parse              = 21;
const long uc_ReleaseItem        = 22;
const long uc_SetCaseSensitivity = 23;
const long uc_SetCodeAddr        = 24;
const long uc_SetDefaultPrecedence = 25;
const long uc_SetDefaultRank     = 26;
const long uc_SetDefaultThread   = 27;
const long uc_SetDefaultType     = 28;
const long uc_SetDefSeparator    = 29;
const long uc_SetErrorNumber     = 30;
const long uc_SetParentThread    = 31;
const long uc_SetSyntaxEqual     = 32;
const long uc_NewThread          = 33;
const long uc_PartialBlockCount  = 34;

const long uc_RegExWordBoundaryChars = 35;
const long uc_RegExEscapeChar = 36;
const long uc_License = 37;
const long uc_License_Demo = 0;
const long uc_License_Free = 2;
const long uc_License_FMP_Lite = 3;
const long uc_License_FMP_Standard = 4;
const long uc_License_LanguageBuilder = 5;
const long uc_License_Beta = 6;

const long uc_AddErrorHandler = 40;

const long uc_GetSymbolTable = 41;

const long uc_GetCallbackData = 50;
const long uc_SetCallbackReturn = 51;

const long uc_GetExpression = 60;
const long uc_GetExpressionType = 61;
const long uc_GetSymbol = 62;
const long uc_GetItem = 63;
const long uc_GetLocation = 64;

const long uc_GetPatternType = 65;
const long uc_GetPatternProperties = 66;

const long uc_GetTrigMode = 67;
const long uc_SetTrigMode = 68;

const long uc_ItemClone = 70;
const long uc_DataConvert = 71;
const long uc_DataCopy = 72;
const long uc_DataAlloc = 72;
const long uc_DataFree = 73;
const long uc_DataReset = 74;

const long uc_SetFPU = 75;
const long uc_SetFPU_HostProg = 76;
const long uc_GetFPU = 77;
const long uc_GetFPU_HostProg = 78;
const long uc_ToggleFPU = 79;
const long uc_ToggleFPU_HostProg = 80;
const long uc_ErrorClear = 85;

const long uc_SetErrorMessage = 91;
const long uc_GetErrorMessage = 92;

const long uc_SerialNumber = 93;
const long uc_SetTurbo = 94;
const long uc_Expand = 95;
const long uc_GetNextItem = 96;
const long uc_GetItemData = 97;
const long uc_SetItemData = 98;
const long uc_GetThreadAddr = 99;
const long uc_GetCodeBlock = 100;

const long uc_Properties = 1;
const long uc_Thread = 2;
const long uc_Address = 3;
const long uc_SymbolName = 4;
const long uc_Rank = 5;
const long uc_Precedence = 6;
const long uc_OverloadHandle = 7;
const long uc_OverloadCount = 8;
const long uc_ByteSize = 9;
const long uc_ArgCount = 10;
const long uc_DataType = 12;
const long uc_Handle = 13;
const long uc_Value = 14;
const long uc_Property = 15;

// Error numbers
const long uc_Err_None                    = 0;
const long uc_Err_Dynamically_Defined     = 1;
const long uc_Err_Syntax_Error            = 2;
const long uc_Err_Undefined_Identifier    = 3;
const long uc_Err_Unknown_Error           = 4;
const long uc_Err_Unrecognized_Pattern    = 5;
const long uc_Err_Unrecognized_Command    = 6;
const long uc_Err_Datatype_Mismatch       = 7;
const long uc_Err_Invalid_Argument_Count  = 8;
const long uc_Err_Invalid_Definition      = 9;
const long uc_Err_CodeBlock_Error         = 10;
const long uc_Err_Undefined_Callback      = 11;
const long uc_Err_ErrMsgAlreadyDefined    = 12;
const long uc_Err_ItemCannotBeModified    = 13;
const long uc_Err_Unrecognized_Member     = 14;

const long uc_Err_Array_Bounds_Exceeded   = 20;
const long uc_Err_Float_Denormal_Operand  = 21;
const long uc_Err_Float_Divide_By_Zero    = 22;
const long uc_Err_Float_Inexact_Result    = 23;
const long uc_Err_Float_Invalid_Operation = 24;
const long uc_Err_Float_Overflow          = 25;
const long uc_Err_Float_Stack_Check       = 26;
const long uc_Err_Float_Underflow         = 27;
const long uc_Err_Integer_Divide_By_Zero  = 28;
const long uc_Err_Integer_Overflow        = 29;
const long uc_Err_Privileged_Instruction  = 30;

// FPU Control Word Settings
const long uc_FPU_Mask_InvalidOp     = 1;
const long uc_FPU_Mask_DenormalOp    = 2;
const long uc_FPU_Mask_ZeroDivide    = 4;
const long uc_FPU_Mask_Overflow      = 8;
const long uc_FPU_Mask_Underflow     = 16;
const long uc_FPU_Mask_PrecisionLoss = 32;

const long uc_FPU_Precision_Single   = 0;
const long uc_FPU_Precision_Double   = 256;
const long uc_FPU_Precision_Extended = 768;

const long uc_FPU_Round_Even         = 0;
const long uc_FPU_Round_Down         = 1024;
const long uc_FPU_Round_Up           = 2048;
const long uc_FPU_Round_Toward       = 3072;

const long ucAbort = 0;
const long ucResume = 1;
const long ucReRaise = 2;

// Data type handler callback action modes
const long uc_TypeHandler_Store = 2;
const long uc_TypeHandler_StringOutput = 3;
const long uc_TypeHandler_Free = 4;
const long uc_TypeHandler_FuncRet = 5;
const long uc_TypeHandler_Copy = 6;
const long uc_TypeHandler_Alloc = 7;
const long uc_TypeHandler_Reset = 8;

const long uc_Callback_Undefined = 0;

// Data Type callbacks
const long uc_Complex     = 1;
const long uc_Currency    = 2;
const long uc_Double      = 3;
const long uc_Extended    = 4;
const long uc_Int8        = 5;
const long uc_Int8u       = 6;
const long uc_Int16       = 7;
const long uc_Int16u      = 8;
const long uc_Int32       = 9;
const long uc_Int32u      = 10;
const long uc_Int64       = 11;
const long uc_Int64u      = 12;
const long uc_Single      = 13;
const long uc_String      = 14;
const long uc_String_LPCSTR= 15;
const long uc_String_Fixed= 16;
const long uc_String_Wide = 17;
const long uc_Stack       = 18;
const long uc_Void        = 19;
const long uc_User_Type   = 20;

const long uc_Convert_Generic = 23;
const long uc_Convert_Double_Extended = 24;
const long uc_Convert_Int8_Extended = 25;
const long uc_Convert_Int8u_Extended = 26;
const long uc_Convert_Int16_Extended = 27;
const long uc_Convert_Int16u_Extended = 28;
const long uc_Convert_Int32_Extended = 29;
const long uc_Convert_Int32u_Extended = 30;
const long uc_Convert_Int64_Extended = 31;
const long uc_Convert_Int64u_Extended = 32;
const long uc_Convert_Single_Extended = 33;
const long uc_Convert_Extended_Double = 37;
const long uc_Convert_Extended_Single = 38;
const long uc_Convert_Extended_Int16u = 39;
const long uc_Convert_Extended_Int32 = 40;
const long uc_Convert_Complex_Extended = 41;
const long uc_Convert_Extended_Complex = 42;

// Default built-In routines
const long uc_Op_Negate_Extended = 43;
const long uc_Op_UnaryPlus_Extended = 44;
const long uc_Op_Percentage_Extended = 45;
const long uc_Op_StrConcat = 46;
const long uc_Op_Add_Int32 = 47;
const long uc_Op_Add_Extended = 48;
const long uc_Op_Add_Complex = 49;
const long uc_Op_Subtract_Int32 = 50;
const long uc_Op_Subtract_Extended = 51;
const long uc_Op_Multiply_Int32 = 52;
const long uc_Op_Multiply_Extended = 53;
const long uc_Op_Multiply_Complex = 54;
const long uc_Op_Divide_Int32 = 55;
const long uc_Op_Divide_Extended = 56;
const long uc_Op_Mod_Int32 = 57;
const long uc_Op_Mod_Extended = 58;
const long uc_Op_PowerOf_ExtToExt = 59;
const long uc_Op_PowerOf_ExtToLng = 60;
const long uc_Op_PowerOf_CplxToCplx = 61;

const long uc_Op_GreaterThan_Extended = 62;
const long uc_Op_GreaterOrEqual_Extended = 63;
const long uc_Op_LessThan_Extended = 64;
const long uc_Op_LessOrEqual_Extended = 65;
const long uc_Op_Equal_Extended = 66;
const long uc_Op_NotEqual_Extended = 67;

const long uc_Op_GreaterThan_Str = 68;
const long uc_Op_GreaterOrEqual_Str = 69;
const long uc_Op_LessThan_Str = 70;
const long uc_Op_LessOrEqual_Str = 71;
const long uc_Op_Equal_Str = 72;
const long uc_Op_NotEqual_Str = 73;

const long uc_Op_Not_Int32 = 74;
const long uc_Op_And_Int32 = 75;
const long uc_Op_Or_Int32 = 76;
const long uc_Op_Xor_Int32 = 77;
const long uc_Op_Eqv_Int32 = 78;
const long uc_Op_Imp_Int32 = 79;

const long uc_Op_AndAlso = 80;
const long uc_Op_OrElse = 81;

const long uc_Op_Factorial = 82;

const long uc_Func_IIf = 83;
const long uc_Func_IIf_Str = 84;

const long uc_Func_Sin = 85;
const long uc_Func_Cos = 86;
const long uc_Func_Tan = 87;
const long uc_Func_Atan = 88;
const long uc_Func_Ceil = 89;
const long uc_Func_Abs_Extended = 90;
const long uc_Func_Abs_Int32 = 91;
const long uc_Func_Exp = 92;
const long uc_Func_Log = 93;
const long uc_Func_Floor = 94;
const long uc_Func_SquareRoot = 95;
const long uc_Func_BaseConvert = 96;

const long uc_Func_ExprHandle = 97;
const long uc_Func_VarPtr = 98;
const long uc_Func_uCalc = 99;
const long uc_Func_uCalcStr = 100;
const long uc_Func_ucAddr = 101;
const long uc_Func_ValueAtAddr = 102;

const long uc_Func_Asc = 103;
const long uc_Func_Chr = 104;

const long uc_Func_For  = 105;
const long uc_Func_Loop = 107;

const long uc_Func_LoadLibrary = 108;
const long uc_Func_GetProcAddress = 109;

const long uc_NullString = 110;
const long uc_True = 111;

const long uc_Func_Peek = 112;
const long uc_Func_Poke = 113;

const long uc_Func_Int = 115;
const long uc_Func_Frac = 116;
const long uc_Func_Sgn = 117;

const long uc_Func_PushData_Stack = 120;
const long uc_Func_PopNum_Stack = 122;
const long uc_Func_PopStr_Stack = 123;
const long uc_Func_ReadHandle_Stack = 124;
const long uc_Func_ReadNum_Stack = 125;
const long uc_Func_ReadStr_Stack = 126;
const long uc_Func_Count_Stack = 127;
const long uc_Op_Concat_Stack = 128;
const long uc_Func_Populate_Stack = 129;

const long uc_Insert_Table = 130;
const long uc_Delete_Table = 131;
const long uc_Index_Table  = 132;
const long uc_Handle_Table = 133;
const long uc_HandleB_Table = 134;

const long uc_Func_StackFormat = 135;

const long uc_SetVariableValue = 140;
const long uc_SetVariableValueAnyType = 140;
const long uc_SetVariableValueLong = 141;
const long uc_SetVariableValueDouble = 142;
const long uc_SetVariableValueExtended = 143;

// Misc other callbacks
const long uc_EvaluateAddr = 150;
const long uc_EvaluateTurboAddr = 151;
const long uc_ExceptionHelper = 152;
const long uc_DefaultErrorHandler = 153;
const long uc_Raise_Error = 154;

// Interpreter addresses
const long uc_Interp_DefHandle     = 160; // Dword Ptr
const long uc_Interp_ExitCommand   = 161; // String Ptr
const long uc_Interp_LastResultNum = 163; // Extended Ptr
const long uc_Interp_LastResultStr = 164; // String Ptr
const long uc_Interp_LineContinue  = 165;
const long uc_Interp_MainPrompt    = 166;
const long uc_Interp_MultiLinePrompt = 167;
const long uc_Interp_Prefix = 168;
const long uc_Interp_Postfix = 169;
const long uc_Interp_Tab = 170;
const long uc_Interp_Insert = 171;

const long uc_Op_Ext_Plus_Complex = 190;
const long uc_Op_Ext_Times_Complex = 191;

const long uc_Func_If_ElseIf = 195;
const long uc_Func_SetSyntaxParams = 196;
const long uc_Func_SpecialUcase = 197;
const long uc_Func_Rand = 198;
const long uc_Func_Batch_Eval = 199;

const long uc_Op_Add_Double = 201;
const long uc_Op_Subtract_Double = 202;
const long uc_Op_Multiply_Double = 203;
const long uc_Op_Divide_Double = 204;
const long uc_Op_Mod_Double = 205;
const long uc_Op_PowerOf_DblToDbl = 206;
const long uc_Op_PowerOf_DblToLng = 207;

const long uc_Func_Sin_Dbl = 208;
const long uc_Func_Cos_Dbl = 209;
const long uc_Func_Tan_Dbl = 210;
const long uc_Func_Atan_Dbl = 211;
const long uc_Func_Ceil_Dbl = 212;
const long uc_Func_Abs_Dbl = 213;
const long uc_Func_Exp_Dbl = 214;
const long uc_Func_Log_Dbl = 215;
const long uc_Func_SquareRoot_Dbl = 216;

const long uc_Convert_Double_Int32 = 218;
const long uc_Convert_Int32_Double = 219;

const long ucTrue  = 1;
const long ucFalse = 0;

const long uc_First = 1;
const long uc_Last  = -1;

// Trig Modes
const long uc_ReturnTrigMode = 0;
const long uc_Radians = 1;
const long uc_Degrees = 2;
const long uc_Grad = 3;

// End Of Constants

#if defined(__BORLANDC__)
   const long uc_Eval = uc_EvalExt;
#else
   const long uc_Eval = uc_EvalDbl;
#endif

typedef DWORD       (CALLBACK* UCALC_)(long, const char*, DWORD, DWORD, DWORD, DWORD);
typedef DWORD       (CALLBACK* UCALC__)(long, long, DWORD, DWORD, DWORD, DWORD);
typedef long double (CALLBACK* UCEVALUATE)(DWORD);
typedef DWORD       (CALLBACK* UCEXPRTHREAD)(DWORD);
typedef long double (CALLBACK* UCPARAM)(DWORD, long);
typedef long        (CALLBACK* UCPARAMCOUNT)(DWORD);
typedef DWORD       (CALLBACK* UCPARAMHANDLE)(DWORD, long);
typedef char*       (CALLBACK* UCPARAMSTR)(DWORD, long);
typedef void        (CALLBACK* UCRETURN)(DWORD, long double);
typedef void        (CALLBACK* UCRETURNSTR)(DWORD, const char*);
typedef void        (CALLBACK* UCSETVARIABLEVALUE)(DWORD, long double);
typedef void        (CALLBACK* UCSETVARIABLEVALUESTR)(DWORD, const char*);
typedef char*       (CALLBACK* UCSTRRETURN)(DWORD);

extern UCALC_ uCalc_;
extern UCALC__ uCalc__;

extern UCEVALUATE ucEvaluate;
extern UCEXPRTHREAD ucExprThread;
extern UCPARAM ucParam;
extern UCPARAMCOUNT ucParamCount;
extern UCPARAMHANDLE ucParamHandle;
extern UCPARAMSTR ucParamStr;
extern UCRETURN ucReturn;
extern UCRETURNSTR ucReturnStr_;
extern UCSETVARIABLEVALUE ucSetVariableValue;
extern UCSETVARIABLEVALUESTR ucSetVariableValueStr;
extern UCSTRRETURN ucStrReturn;

extern long ucInitialized;
extern long ucLoadLibFailed;
extern DWORD ucExtended, ucString, ucLPCSTR, ucFixedString, ucDouble, ucSingle;
extern DWORD ucLong, ucDword, ucComplex, ucByte, ucTable, ucVoid, ucStack;
extern DWORD ucSortedList;
extern DWORD ucDefaultErrorHandler;

#endif /* __UCALC__ */

