unit Spreadsheet;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   StdCtrls, Grids, uCalcDlp;

type
   TForm1 = class(TForm)
      txtCellName: TEdit;
      txtFormula: TEdit;
      Label1: TLabel;
      Grid1: TStringGrid;
      btnSetColumnName: TButton;
      txtColumn: TEdit;
      Label3: TLabel;
      Label2: TLabel;
      procedure Grid1Click(Sender: TObject);
      procedure btnSetColumnNameClick(Sender: TObject);
      procedure txtCellNameKeyPress(Sender: TObject; var Key: Char);
      procedure txtFormulaChange(Sender: TObject);
      procedure txtFormulaExit(Sender: TObject);
      procedure FormCreate(Sender: TObject);
      procedure txtFormulaKeyPress(Sender: TObject; var Key: Char);
   private
      { Private declarations }
   public
      { Public declarations }
   end;

const
   MaxRow = 10;
   MaxCol = 5;
   ArraySpace = 50; // ArraySpace = MaxRow*MaxCol;
var
   Form1: TForm1;
   UpdatingDepCells: Boolean;
   CellRow, CellCol, CurrentCell: Longint;
   DepIndex: Longint = 0;
   NewCellName: String;
   ColumnName: array[0..MaxCol] of string;
   CellEq: array[0..ArraySpace] of string;
   CellName: array[0..MaxRow, 0..MaxCol] of string;
   CellArray: array[0..ArraySpace] of Extended;
   DepNext, DepList, DepStart, FormulaPtr: array[0..ArraySpace] of Longint;

   procedure CellArrayValue(Expr: Longword); stdcall;
   procedure SetCellValue(cRow, cCol: Longint; CellContent: String);
   procedure UpdateDepCells(GridCell: Longint; CellValue: Extended);
   procedure SetColName(ColumnName: String; ArrayCol: Longint);
   procedure ProcessCellNames(Eq: String);

implementation

{$R *.DFM}

procedure TForm1.Grid1Click(Sender: TObject);
var
   CellAddress: Longint;
begin
   CellRow := Grid1.Row;
   CellCol := Grid1.Col;
   CellAddress := (CellCol - 1) * MaxRow + CellRow;

   if Copy(CellEq[CellAddress], 1, 1) = '='
   then txtFormula.Text := CellEq[CellAddress]
   else txtFormula.Text := Grid1.Cells[CellCol, CellRow];

   txtColumn.Text := ColumnName[CellCol];
   txtCellName.Text := CellName[CellRow, CellCol];

   if Length(txtCellName.Text) = 0
   then txtCellName.Text := txtColumn.Text + IntToStr(CellRow);

   txtFormula.SetFocus;

   if txtFormula.Text = '' then txtFormula.Text := '=';
   txtFormula.SelStart := Length(txtFormula.Text);   
end;

procedure TForm1.btnSetColumnNameClick(Sender: TObject);
begin
   ColumnName[CellCol] := txtColumn.Text;
   SetColName(ColumnName[CellCol], CellCol);
end;

procedure TForm1.txtCellNameKeyPress(Sender: TObject; var Key: Char);
begin
   If Key = Chr(13) then // 13 = Enter
   begin
      ucDefineSyntax(txtCellName.Text + '::=' + ColumnName[CellCol] + '(' + IntToStr(CellRow) + ')');
      CellName[CellRow, CellCol] := txtCellName.Text;
      Grid1.SetFocus
   end;
end;

procedure TForm1.txtFormulaChange(Sender: TObject);
begin
   Grid1.Cells[CellCol, CellRow] := txtFormula.Text;
end;

procedure TForm1.txtFormulaExit(Sender: TObject);
begin
   if txtFormula.Text = '='
   then txtFormula.Text := ''
   else SetCellValue(CellRow, CellCol, txtFormula.Text);
end;

procedure CellArrayValue(Expr: Longword); stdcall;
var
   CellAddress, iIndex, TmpIndex, cRow, cCol: Longint;
begin
   cRow := Trunc(ucParam(Expr, 1));
   cCol := Trunc(ucParam(Expr, 2));
   CellAddress := (cCol - 1) * MaxRow + cRow;
   TmpIndex := 0;

   ucReturn(Expr, CellArray[CellAddress]);

   if UpdatingDepCells = True then Exit;

   if DepStart[CellAddress] = 0 then
   begin
      if DepIndex = 0 then DepIndex := DepIndex + 1;
      DepStart[CellAddress] := DepIndex;
      DepNext[DepIndex] := -1;
   end
   else
   begin
      iIndex := DepStart[CellAddress];
      repeat
         if DepList[iIndex] = CurrentCell then Exit;
         if DepNext[iIndex] = -1 then
         begin
            DepNext[iIndex] := DepIndex;
            DepNext[DepIndex] := -1;
            Break;
         end;
         TmpIndex := iIndex;
         iIndex := DepNext[iIndex];
      until DepNext[TmpIndex] = -1;
   end;

   DepList[DepIndex] := CurrentCell;
   DepIndex := DepIndex + 1;
end;

procedure SetCellValue(cRow, cCol: Longint; CellContent: String);
var
   EqPtr: Longint;
   CellValue: Extended;
begin
   CurrentCell := (cCol - 1) * MaxRow + cRow;
   if Length(CellContent) = 0 then Exit;
   if CellContent[1] = '=' then
   begin
      CellContent[1] := ' ';
      ProcessCellNames(CellContent);
      EqPtr := ucParse(CellContent);
      CellValue := ucEvaluate(EqPtr);
      FormulaPtr[CurrentCell] := EqPtr;
      CellContent[1] := '=';
      CellEq[CurrentCell] := CellContent;
   end
   else
      CellValue := StrToFloat(CellContent);

   UpdatingDepCells := True;
   UpdateDepCells(CurrentCell, CellValue);
   UpdatingDepCells := False;
end;

procedure UpdateDepCells(GridCell: Longint; CellValue: Extended);
var
   iIndex, DepCell, TmpRow, TmpCol: Longint;
begin
   TmpCol := (GridCell - 1) div MaxRow + 1;
   TmpRow := GridCell - (TmpCol - 1) * MaxRow;
   Form1.Grid1.Cells[TmpCol, TmpRow] := FloatToStr(CellValue);
   CellArray[GridCell] := CellValue;

   iIndex := DepStart[GridCell];
   if iIndex <> 0 then
   begin
      while iIndex <> -1 do
      begin
         DepCell := DepList[iIndex];
         CellValue := ucEvaluate(FormulaPtr[DepCell]);
         UpdateDepCells(DepCell, CellValue);
         iIndex := DepNext[iIndex];
      end;
   end;
end;

procedure SetColName(ColumnName: String; ArrayCol: Longint);
begin
   ucDefineFunction(ColumnName + '(x)=Cell(x,' + IntToStr(ArrayCol) + ' )');
end;

procedure ProcessCellNames(Eq: String);
begin
   ucEval(Eq);

   while Length(NewCellName) > 0 do
   begin
      ucDefineSyntax(NewCellName + '::=' + NewCellName[1] + '(' + Copy(NewCellName, 2, Length(NewCellName)-1) + ')');
      NewCellName := '';
      ucEval(Eq);
   end;
end;

function MyErrorHandler(t: Longword): Longword; stdcall;
begin
   if ucError(t) = uc_Err_Undefined_Identifier
   then NewCellName := ucErrSymbol(t);

   MyErrorHandler := ucAbort;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
   x: Longint;
   Alpha: String;
begin
   ucDefineFunction('Native: Cell(Row, Col)', @CellArrayValue);
   uCalc(uc_AddErrorHandler, 0, Longword(@MyErrorHandler));

   Alpha := 'ABCDEFGHIJK';

   for x := 1 to MaxRow do
      Grid1.Cells[0, x] := IntToStr(x);

   for x := 1 To MaxCol do
   begin
      Grid1.Cells[x, 0] := Copy(Alpha, x, 1);
      ColumnName[x] := Copy(Alpha, x, 1);
      SetColName(ColumnName[x], x);
   end;

   Grid1.Row := 1;
   Grid1.Col := 1;
   Show;
   Grid1Click(Sender);
end;

procedure TForm1.txtFormulaKeyPress(Sender: TObject; var Key: Char);
begin
   if Key = #13 then SetCellValue(CellRow, CellCol, txtFormula.Text);
end;

end.
